"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prefixTransition = prefixTransition;

function prefixTransition(prefix, transition) {
  if (prefix instanceof RegExp && prefix.toString()[1] !== "^") {
    throw new Error("Prefix regexp should start with ^");
  }

  const hasPrefix = pathname => {
    if (prefix instanceof RegExp) {
      return prefix.test(pathname);
    } else {
      return pathname.startsWith(prefix);
    }
  };

  const removePrefix = pathname => {
    if (prefix instanceof RegExp) {
      return pathname.replace(prefix, "");
    } else {
      return pathname.slice(prefix.length);
    }
  };

  return async function prefixedTransition(location) {
    const {
      pathname,
      ...rest
    } = location;

    if (!hasPrefix(pathname)) {
      return null;
    }

    return transition({ ...rest,
      pathname: removePrefix(pathname)
    });
  };
}